package Jbroker;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A dialog box to authentificate.
 * @version Java Fast Broker 1.0
 * @author Valentin Zaharescu & Bogdan Stanca<br>
 * Copyright (c)2000 all rights reserved<br>
 */
public class JbAuthentificationDialog extends JDialog
{
    private JPanel mainPanel = null;

    private JTextField emailField = null;
    private JPasswordField passwordField = null;

    private JButton okayButton = null;
    private JButton cancelButton = null;

    private boolean result = false;


    /**
     * Construct a dialog.
     */
    public JbAuthentificationDialog(Frame parent)
    {
        super(parent, "Beglaubigung", true);

        // Dialog is not resizable.
        setResizable(false);
	setSize(300, 250);
	setModal(true);
	mainPanel = new JPanel() {
    		public  void paintComponent(Graphics g) {
		    if (g != null) {
			ImageIcon tmpImage = new ImageIcon(JbDefinition.AuthentificationImage);
			g.drawImage(tmpImage.getImage(), 0, 0, mainPanel.getWidth(), mainPanel.getHeight(), this);
			super.paintComponent(g);
		    }
		}};
	mainPanel.setOpaque(false);

        // Set close handler for window.
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent event)
		{
		    doCancel();
		}
	    });


        // Layout components.
        getContentPane().add(mainPanel);
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(createEditPanel(), BorderLayout.NORTH);
        mainPanel.add(createButtonPanel(), BorderLayout.SOUTH);
    }


    /**
     * Return the result of the dialog.
     */
    public boolean isOkay()
    {
        return result;
    }


    /**
     * Return the email.
     */
    public String getEmail()
    {
        return emailField.getText();
    }

    /**
     * Set the email
     */
    public void setEmail(String email)
    {
        emailField.setText(email);
    }



    /**
     * Return the password
     */
    public int getPassword()
    {
    	try {
	    return Integer.parseInt(new String(passwordField.getPassword()).trim());
        }catch(NumberFormatException e){};
        return -1;
    }

    /**
     * Set the password
     */
    public void setpassword(int password)
    {
        passwordField.setText(new Integer(password).toString());
    }



    /**
     * Closes the dialog with "true" result.
     */
    protected void doOkay()
    {
    	if (emailField.getText().length() == 0) {
	    result = false;
	    return;
	}

    	if (new String(passwordField.getPassword()).length() == 0) {
	    result = false;
	    return;
	}

        result = true;
        setVisible(false);
        dispose();
    }

    /**
     * Closes the dialog with "false" result.
     */
    protected void doCancel()
    {
        result = false;
        setVisible(false);
        dispose();
    }

    /**
     * Creates the edit panel.
     */
    protected JPanel createEditPanel()
    {
	JPanel panel = new JPanel(new GridLayout(2,2));
	panel.setOpaque(false);
	JLabel ulabel= null, plabel=null;

	panel.add(ulabel = new JLabel("Klient:"));
	ulabel.setForeground(Color.white);
	ulabel.setHorizontalAlignment(JLabel.CENTER);

	panel.add(emailField = new JTextField(20));

	panel.add(plabel = new JLabel("Kennwort:"));
	plabel.setForeground(Color.white);
	plabel.setHorizontalAlignment(JLabel.CENTER);

	panel.add(passwordField = new JPasswordField(20));
	return panel;
    }


    /**
     * Creates the button panel.
     */
    protected JPanel createButtonPanel()
    {
        JPanel panel = new JPanel();
	panel.setOpaque(false);

        panel.add(okayButton = new JButton("Na gut"));
        panel.add(cancelButton = new JButton("Spter"));

        // Add some action listeners.
        okayButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    doOkay();
		}
	    });

        cancelButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    doCancel();
		}
	    });

        return panel;
    }
}
