package Jbroker;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;

/**
 * Chart Panel
 * @version Java Fast Broker 1.0
 * @author Valentin Zaharescu & Bogdan Stanca<br>
 * Copyright (c)2000 all rights reserved<br>
 */
public class JbChart extends JPanel{

    private int wkn = 0;
    private int months = 1;
    private int compareToWKN = 0;
    private double min, max, medium;
    private int redCount = 0;
    private int redCompareToCount = 0;
    private int kursAvg = 20;
    private double[] redY;
    private double[] redCompareToY;
    private String aktieName;
    private String compareToAktieName;

    private final int border = 5; // border width
    private final int leftBorder = 70; // left border width

/**
 * Constructor
 */
    public JbChart() {
	super();
	setBackground(new Color(80, 150, 70));
    }

/**
 * Constructor
 */
    public JbChart(int wkn) {
	super();
	this.wkn = wkn;

	setBackground(new Color(80, 150, 70));
	readData();
    }

    public Dimension getPreferredSize() {
	return new Dimension(600, 300);
    }

    public int getAvgOver() {
    	return kursAvg;
    }

    public void setAvgOver(int avg) {
    	if ((avg > 1) && (avg < 101))
	    this.kursAvg = avg;
	else
	    this.kursAvg = 20;
	this.paintComponent(this.getGraphics());
    }

    public void setWKN(int wkn) {
	this.wkn = wkn;
	readData();
    }

    public int getCompareToWKN() {
	return compareToWKN;
    }

    public void setCompareToWKN(int wkn) {
	this.compareToWKN = wkn;
	readData();
    }

    public void setPeriod(int months) {
	this.months = months;
	readData();
    }

/**
 * Read all data from the database
 */
    public void readData() {

	if (wkn == compareToWKN)
	    compareToWKN = 0;

	// read wkn Name
	ResultSet res = JbSQL.query("select name from aktie where (wkn="+wkn+")");
	try{
	    if (res.next())
		aktieName = res.getString(1);
	    else
		aktieName = new String("Unknown");
	}catch(SQLException e){
	    e.printStackTrace ();
	}

	redCount=0;
	try{
	    // count records for wkn
	    if (months > 12)
		res = JbSQL.query("select count(*) from kurs where (wkn="+wkn+")");
	    else
		res = JbSQL.query("select count(*) from kurs where (wkn="+wkn+" and Datum>=ADD_MONTHS(\'"+JbSQL.today()+"\', -"+months+"))");
	    while (res.next())
		redCount = res.getInt(1);
	}catch(SQLException e){
	    e.printStackTrace ();
	}

	min=1E100;
	max=-1E100;

	redY = new double[redCount];

	// read records for wkn
	if (months > 12)
	    res = JbSQL.query("select tageskurs from kurs where (wkn="+wkn+") order by datum");
	else
	    res = JbSQL.query("select tageskurs from kurs where (wkn="+wkn+" and Datum>=ADD_MONTHS(\'"+JbSQL.today()+"\', -"+months+")) order by datum");
	try{
	    int i=0;
	    double s=0;
	    while(res.next()){
		redY[i]=Double.parseDouble(res.getString(1).trim());
		if(redY[i]<min)
		    min=redY[i];
		if(redY[i]>max)
		    max=redY[i];
		s+=redY[i];
		i++;
	    }

	    medium=s/redCount;

	}catch(SQLException e){
	    e.printStackTrace ();
	}

	// read compareToWKN Name
	res = JbSQL.query("select name from aktie where (wkn="+compareToWKN+")");
	try{
	    if (res.next())
		compareToAktieName = res.getString(1);
	    else
		compareToAktieName = new String("Unknown");
	}catch(SQLException e){
	    e.printStackTrace ();
	}

	// count records for compareToWKN
	if (months > 12)
	    res = JbSQL.query("select count(*) from kurs where (wkn="+compareToWKN+")");
	else
	    res = JbSQL.query("select count(*) from kurs where (wkn="+compareToWKN+" and Datum>=ADD_MONTHS(\'"+JbSQL.today()+"\', -"+months+"))");
	redCompareToCount=0;
	try{
	    while(res.next())
		redCompareToCount = res.getInt(1);
	}catch(SQLException e){
	    e.printStackTrace ();
	}

	redCompareToY = new double[redCompareToCount];

	// read records for compareToWKN
	if (months > 12)
	    res = JbSQL.query("select tageskurs from kurs where (wkn="+compareToWKN+") order by datum");
	else
	    res = JbSQL.query("select tageskurs from kurs where (wkn="+compareToWKN+" and Datum>=ADD_MONTHS(\'"+JbSQL.today()+"\', -"+months+")) order by datum");
	try{
	    int i=0;
	    while(res.next()){
		redCompareToY[i]=res.getDouble(1);
		if(redCompareToY[i]<min)
		    min=redCompareToY[i];
		if(redCompareToY[i]>max)
		    max=redCompareToY[i];
		i++;
	    }

    	}catch(SQLException e){
	    e.printStackTrace ();
	}

	if ((redCount > 0) && (redCompareToCount > 0)) {
	    // minCount = minimum(redCompareToCount, redCount)
	    int minCount = (int)(((redCompareToCount+redCount)-(redCompareToCount-redCount))/2.0);

	    for(int i=0;i<minCount;i++) {
		double aux = redY[i] - redCompareToY[i];
		if(aux<min)
		    min=aux;
		if(aux>max)
		    max=aux;
	    }
	}

	this.paintComponent(this.getGraphics());
    }

/**
 * Draws the graph
 */
     public void paintComponent(Graphics g) {
	if (g == null)
	    return;

	super.paintComponent(g);

	int w = getSize().width-border-leftBorder; // width of the drawable area
	int h = getSize().height-border*2; // height of the drawable area

	g.setColor(Color.black);
	g.drawRect(leftBorder, border, w, h);

	if (redCount == 0)
	    return;

	// starting date
	if (months <= 12) {
	    try{
		ResultSet res = JbSQL.query("select TO_CHAR(min(Datum),\'DD-MON-YYYY\') from kurs where (wkn="+wkn+" and Datum>=ADD_MONTHS(\'"+JbSQL.today()+"\', -"+months+"))");
		if (res.next())
		    g.drawString(res.getString(1).trim(), leftBorder+5, h);
	    }catch(SQLException e){
		e.printStackTrace ();
	    }
	}
	else
	    g.drawString(JbSQL.toSQLDate(JbDefinition.setFoundationDate(null)), leftBorder+5, h);
	// ending date
	g.drawString(JbSQL.today(), w-20, h);

	// zero value
	g.drawString(JbDefinition.sprintf(0.0000,3), border, border+(int)(h-h*(-min)/(max-min)));
	// zero line
	g.drawLine(leftBorder+2, border+(int)(h-h*(-min)/(max-min)),
		   getSize().width-border, border+(int)(h-h*(-min)/(max-min)) );

	// max value
	g.drawString(JbDefinition.sprintf(max,3), border, border+10);

	// medium line value
	g.drawString(JbDefinition.sprintf(medium,3), border, 
		     border+(int)(h-h*(medium-min)/(max-min) ) );

	// min value
	g.drawString(JbDefinition.sprintf(min,3), border, h);

	int[] vx1 = new int[redCount];
	int[] vy1 = new int[redCount]; // wkn
	int[] vy2 = new int[redCount]; // wkn mediated over kursAvg values

	for(int i=0;i<redCount;i++)
	    {
		vx1[i] = leftBorder+i*w/redCount;
		vy1[i] = border+(int)(h*(1.0 - ( redY[i] - min )/(max-min)));

		int avg = 0;
		double aux=0.0;
		for(int j=i-(int)(kursAvg/2.0);j<i+(int)(kursAvg/2.0);j++) {
		    if ((j >= 0) && (j < redCount)) {
			aux += redY[j];
			avg++;
		    }
		    vy2[i] = border+(int)(h*(1.0 - ( aux/avg - min )/(max-min)));
		}
	    }

	g.setColor(Color.white);
	g.drawString(aktieName, leftBorder, border+10); // print wkn Name Legend
	g.drawPolyline(vx1, vy1, redCount); // draw wkn

	g.setColor(Color.green); // print wkn averange Legend
	g.drawString(aktieName+" Durchschnitt auf "+kursAvg+" Tage", leftBorder, border+20);
	g.drawPolyline(vx1, vy2, redCount); // draw averanged wkn over kursAvg values

	g.setColor(new Color(255, 140, 18)); // print wkn medium Legend
	g.drawString(aktieName+" Durchschnittwert Linie", leftBorder, border+30);
	g.drawLine(leftBorder+2, border+(int)(h-h*(medium-min)/(max-min)),
		   getSize().width-border, border+(int)(h-h*(medium-min)/(max-min)) );


	if (redCompareToCount == 0)
	    return;

	int[] vx3 = new int[redCompareToCount];
	int[] vy3 = new int[redCompareToCount]; // compareToWKN

	for(int i=0;i<redCompareToCount;i++)
	    {
		vx3[i] = leftBorder+i*w/redCompareToCount;
		vy3[i] = border+(int)(h*(1.0 - ( redCompareToY[i] - min )/(max-min)));
	    }

	// minCount = minimum(redCompareToCount, redCount)
	int minCount = (int)(((redCompareToCount+redCount)-(redCompareToCount-redCount))/2.0);

	// maxCount = minimum(redCompareToCount, redCount)
	int maxCount = (int)(((redCompareToCount+redCount)+(redCompareToCount-redCount))/2.0);

	int[] vx4 = new int[minCount];
	int[] vy4 = new int[minCount]; // difference between wkn and compareToWKN

	for(int i=0;i<minCount;i++) {
	    vx4[i] = leftBorder+i*w/maxCount;
	    vy4[i] = border+(int)(h*(1.0 - ( redY[i] - redCompareToY[i] - min )/(max-min)));
	}

	g.setColor(Color.black);	// print compareToWKN Name Legend
	g.drawString(compareToAktieName, leftBorder, border+40);
	g.drawPolyline(vx3, vy3, redCompareToCount); // draw compareToWKN

	g.setColor(Color.blue);	// print difference Legend
	g.drawString("Diferenz (erster - zweiter)", leftBorder, border+50);
	g.drawPolyline(vx4, vy4, minCount); // draw difference between wkn and compareToWKN
    }

}
