package Jbroker;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Main panel class (the one with many buttons on it)
 * @version Java Fast Broker 1.0
 * @author Valentin Zaharescu & Bogdan Stanca<br>
 * Copyright (c)2000 all rights reserved<br>
 */
public class JbMainPanel extends JPanel{

    final private JPanel centerPanel = new JPanel(new GridLayout(3, 4)) ;

    private JButton kontoStandButton;
    private JButton ueberweisungButton;
    private JButton kreditLimitButton;
    private JButton depotStandButton;

    private JButton kaufenButton;
    private JButton verkaufenButton;

    private JButton aktienVergleichButton;
    private JButton gewinStandButton;
    private JButton grossterGewinButton;
    private JButton todaysOfferButton;

    private JbStockGainTable stockGainTable;
    private JbUeberweisungDialog ueberweisungDialog;
    private JbKontoStand kontoStand;
    private boolean tableIsOn = false;
    private JbCreditLimitDialog creditLimitDialog = null;
    private JbTable table = null;
    private HashMap aktienHM = null;
    private JbOrderDialog buyFrame = null;
    private JbOrderDialog sellFrame = null;
    private JbChartDialog chartDialog = null;
    private JbWertSteigerungDialog wertSteigerungDialog = null;
    private JbBestDialog bestDialog = null;

    private String userName;
    private int kundeID, depotID, kontoID;

/**
 * Constructor
 */
    public JbMainPanel(int kundeID) {
	super();
	this.kundeID = kundeID;

	try
	    {
		ResultSet Res = JbSQL.query("select Name, depotID, kontoID from kunde where kID=\'"+kundeID+"\'");
		while(Res.next()){
		    userName = Res.getString(1).trim();
		    depotID = Integer.parseInt(Res.getString(2).trim());
		    kontoID = Integer.parseInt(Res.getString(3).trim());
		}
	    }
	catch (Exception e)
	    {
		e.printStackTrace ();
	    }

	// Set layout manager.
        setLayout(new BorderLayout());

	add(centerPanel, BorderLayout.CENTER);

	centerPanel.add(kontoStandButton = new JButton("Kontostand"));
	kontoStandButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    createKontoStandDialog();
    		}});

	centerPanel.add(ueberweisungButton = new JButton("berweisung"));
	ueberweisungButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    createUeberweisungDialog();
    		}});

	centerPanel.add(kreditLimitButton = new JButton("Kreditlimit"));
	kreditLimitButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    createCreditLimitDialog();
    		}});

	centerPanel.add(aktienVergleichButton = new JButton("Graphik"));
	aktienVergleichButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    createChartDialog();
   		}});

	centerPanel.add(depotStandButton = new JButton("Depotstand"));
	depotStandButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    tableIsOn = true;
		    createDepotStandDialog();
    		}});

	centerPanel.add(todaysOfferButton = new JButton("Depot ausfrlich"));
	todaysOfferButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    createErsteWahlDialog();
   		}});

	centerPanel.add(kaufenButton = new JButton("Kaufen"));
	kaufenButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    createBuyDialog();
   		}});
	centerPanel.add(verkaufenButton = new JButton("Verkaufen"));
	verkaufenButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    if (JbSQL.keineAktienImDepot(depotID)) {
			JOptionPane.showMessageDialog(null,
						      "Keine Aktien im Depot vorhanden!",
						      "Verkaufen", JOptionPane.ERROR_MESSAGE);
			return;
		    }
		    createSellDialog();
   		}});

	centerPanel.add(gewinStandButton = new JButton("Gewin seit kauf"));
	gewinStandButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    if (JbSQL.keineAktienImDepot(depotID)) {
			JOptionPane.showMessageDialog(null,
						      "Keine Aktien im Depot vorhanden!",
						      "Fehler", JOptionPane.ERROR_MESSAGE);
			return;
		    }
		    createWertSteigerungDialog();
   		}});

	centerPanel.add(grossterGewinButton = new JButton("Grsster Gewin"));
	grossterGewinButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    createBestDialog();
   		}});

	centerPanel.add(new JLabel("JayBroker Datum", JLabel.RIGHT));
	centerPanel.add(new JLabel(JbSQL.today(), JLabel.CENTER));
    }

/**
 * Constructs the Chart Dialog
 */
    public void createChartDialog() {
	if (chartDialog != null) {
	    chartDialog.setVisible(false);
	    chartDialog.dispose();
	}
	chartDialog = new JbChartDialog("Aktien Vergleich");
	chartDialog.setVisible(true);
    }

/**
 * Constructs the buy Dialog
 */
    public void createBuyDialog() {
	if (buyFrame != null) {
	    buyFrame.setVisible(false);
	    buyFrame.dispose();
	}

	if (aktienHM != null)
	    aktienHM = JbSQL.createAktienHashMap();

	buyFrame = new JbOrderDialog(true, kundeID, depotID, kontoID);
	buyFrame.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent event)
		{
		    if (table != null)
			createDepotStandDialog();
		}
	    });
	buyFrame.setVisible(true);
	buyFrame.pack();
    }

/**
 * Constructs the sell Dialog
 */
    public void createSellDialog() {
	if (sellFrame != null) {
	    sellFrame.setVisible(false);
	    sellFrame.dispose();
	}

	if (aktienHM != null)
	    aktienHM = JbSQL.createOwnAktienHashMap(depotID);

	sellFrame = new JbOrderDialog(false, kundeID, depotID, kontoID);
	sellFrame.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent event)
		{
		    if (table != null)
			createDepotStandDialog();
		}
	    });
	sellFrame.setVisible(true);
	sellFrame.pack();
    }

    /**
     * Constructs the balance dialog
     */
    public void createKontoStandDialog() {
	if (kontoStand != null) {
	    kontoStand.setVisible(false);
	    kontoStand.dispose();
	}
	kontoStand = new JbKontoStand(kontoID, depotID);
	kontoStand.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent event)
		{
				// something to be happenning here on tableClose
		}
	    });
	kontoStand.pack();
	kontoStand.setVisible(true);
    }

    /**
     * Constructs the depot status dialog
     */
    public void createDepotStandDialog() {
	if (table != null) {
	    table.setVisible(false);
	    table.dispose();
	}
	if (tableIsOn) {
	    table = new JbTable(new String("Depotinhalt"),
				new String("select a1.wkn, a1.Name, a2.Anzahl, a1.AktKurs from aktie a1, "+
					   "(select d.WKN, sum(d.Anzahl) Anzahl "+
					   "from DepotInhalt d where d.DepotID=\'"+depotID+"\' group by d.WKN) a2 "+
					   "where a1.WKN=a2.WKN order by a1.Name"),
				new String[]{"WKN","Name","Anzahl","AktKurs"});
	    table.addWindowListener(new WindowAdapter() {
		    public void windowClosed(WindowEvent event)
		    {
			// something to be happenning here on tableClose
		    }
		});
	    table.pack();
	    table.setVisible(true);
	}
    }

    /**
     * Constructs the money transfer dialog
     */
    public void createUeberweisungDialog() {
	if (ueberweisungDialog != null) {
	    ueberweisungDialog.setVisible(false);
	    ueberweisungDialog.dispose();
	}
	ueberweisungDialog = new JbUeberweisungDialog(kontoID);
	ueberweisungDialog.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent event)
		{
				// something to be happenning here on tableClose
		}
	    });
	ueberweisungDialog.pack();
	ueberweisungDialog.setVisible(true);
    }

    /**
     * Constructs the credit limit negociation Dialog
     */
    public void createCreditLimitDialog() {
	if (creditLimitDialog != null) {
	    creditLimitDialog.setVisible(false);
	    creditLimitDialog.dispose();
	}
	creditLimitDialog = new JbCreditLimitDialog(kontoID, depotID);
	creditLimitDialog.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent event)
		{
		    // something to be happenning here on winClose
		}
	    });
	creditLimitDialog.pack();
	creditLimitDialog.setVisible(true);
    }

    /**
     * Constructs the gain Dialog
     */
    public void createErsteWahlDialog() {
	if (stockGainTable != null) {
	    stockGainTable.setVisible(false);
	    stockGainTable.dispose();
	}
	stockGainTable = new JbStockGainTable(kundeID, depotID);
	stockGainTable.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent event)
		{
		    // something to be happenning here on tableClose
		}
	    });
	stockGainTable.pack();
	stockGainTable.setVisible(true);
    }

    /**
     * Constructs the gain growth Dialog
     */
    public void createWertSteigerungDialog() {
	if (wertSteigerungDialog != null) {
	    wertSteigerungDialog.setVisible(false);
	    wertSteigerungDialog.dispose();
	}
	wertSteigerungDialog = new JbWertSteigerungDialog(kundeID, depotID, kontoID);
	wertSteigerungDialog.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent event)
		{
		    // something to be happenning here on Close
		}
	    });
	wertSteigerungDialog.pack();
	wertSteigerungDialog.setVisible(true);
    }

    /**
     * Constructs the best aktien Dialog
     */
    public void createBestDialog() {
	if (bestDialog != null) {
	    bestDialog.setVisible(false);
	    bestDialog.dispose();
	}
	bestDialog = new JbBestDialog(kundeID, depotID, kontoID);
	bestDialog.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent event)
		{
		    // something to be happenning here on Close
		}
	    });
	bestDialog.pack();
	bestDialog.setVisible(true);
    }

    /**
     * "Destructor" (closes all the windows at logout time)
     */
    public void deleteProcesses() {
	tableIsOn = false;
	if (table != null) {
	    table.setVisible(false);
	    table.dispose();
	}
	if (stockGainTable != null) {
	    stockGainTable.setVisible(false);
	    stockGainTable.dispose();
	}
	if (wertSteigerungDialog != null) {
	    wertSteigerungDialog.setVisible(false);
	    wertSteigerungDialog.dispose();
	}
	if (bestDialog != null) {
	    bestDialog.setVisible(false);
	    bestDialog.dispose();
	}
	if (creditLimitDialog != null) {
	    creditLimitDialog.setVisible(false);
	    creditLimitDialog.dispose();
	}
	if (buyFrame != null) {
	    buyFrame.setVisible(false);
	    buyFrame.dispose();
	}
	if (sellFrame != null) {
	    sellFrame.setVisible(false);
	    sellFrame.dispose();
	}
	if (chartDialog != null) {
	    chartDialog.setVisible(false);
	    chartDialog.dispose();
	}
	if (kontoStand != null) {
	    kontoStand.setVisible(false);
	    kontoStand.dispose();
	}
	if (ueberweisungDialog != null) {
	    ueberweisungDialog.setVisible(false);
	    ueberweisungDialog.dispose();
	}
    }
}
