package Jbroker;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import java.sql.*;

/**
 * Order Dialog
 * @version Java Fast Broker 1.0
 * @author Valentin Zaharescu & Bogdan Stanca<br>
 * Copyright (c)2000 all rights reserved<br>
 */
public class JbOrderDialog extends JFrame
{
    private final String noComparation = "Keinem";

    private JComboBox comboAktie;
    private int depotID;
    private int kundeID;
    private int kontoID;
    private double total = 0;
    private boolean buy;
    private JTextField anzahl;
    private HashMap aktienHM;

    private JLabel kontoStandLabel;
    private JLabel kreditLimitLabel;
    private JLabel kreditLimitEURLabel;
    private JLabel kursLabel;
    private JLabel totalLabel;

/**
 * Constructor
 */
    public JbOrderDialog(boolean buy, int kundeID, int depotID, int kontoID) {
	super(buy ? "Aktien kaufen" : "Aktien verkaufen");
	this.buy = buy;
	this.kundeID = kundeID;
	this.depotID = depotID;
	this.kontoID = kontoID;

	createDialog();
    }

/**
 * Set up the Dialog
 */
    private void createDialog() {
	if (buy) {
	    comboAktie = JbSQL.createAktienJComboBox();
	    aktienHM = JbSQL.createAktienHashMap();
	} else {
	    comboAktie = JbSQL.createOwnAktienJComboBox(depotID);
	    aktienHM = JbSQL.createOwnAktienHashMap(depotID);
	}
	getContentPane().setLayout(new GridLayout(8, 2));

	getContentPane().add(new JLabel("Kontostand (EUR / DM)   "));
	getContentPane().add(kontoStandLabel = new JLabel("0/0"));

	getContentPane().add(new JLabel("Kreditlimit (% Depot)  "));
	getContentPane().add(kreditLimitLabel = new JLabel("0"));

	getContentPane().add(new JLabel("Kreditlimit (EUR / DM)  "));
	getContentPane().add(kreditLimitEURLabel = new JLabel("0"));

	kontoStandLabel.setText(JbDefinition.betragEUR_DM(new Double(JbSQL.getKundeKontoStand(kontoID)).toString()));
	kreditLimitLabel.setText(JbDefinition.sprintf(JbSQL.getKundeKreditLimit_prozent(kontoID),3));
	kreditLimitEURLabel.setText(JbDefinition.betragEUR_DM(new Double(JbSQL.getKundeKreditLimit_EUR(depotID, kontoID)).toString()));

	getContentPane().add(new JLabel("Aktie"));
	getContentPane().add(comboAktie);

	getContentPane().add(new JLabel("Kurs (EUR / DM)"));
	getContentPane().add(kursLabel = new JLabel("0/0"));

	anzahl = new JTextField(10);
	anzahl.setText("100");
	anzahl.selectAll();

	getContentPane().add(new JLabel("Anzahl"));
	getContentPane().add(anzahl);
	anzahl.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    kursLabel.setText(new Double(((JbAktie)(aktienHM.get((String)(comboAktie.getSelectedItem())))).getAktKurs()).toString());
		    try {
			totalLabel.setText(new Double(Double.parseDouble(kursLabel.getText().trim())*
						      Double.parseDouble(anzahl.getText().trim())).toString());
			
			total = Double.parseDouble(totalLabel.getText().trim());

			kursLabel.setText(JbDefinition.betragEUR_DM(kursLabel.getText()));
			totalLabel.setText(JbDefinition.betragEUR_DM(totalLabel.getText()));
		    } catch(Exception ex) {};
   		}});

	getContentPane().add(new JLabel("Total (EUR / DM)"));
	getContentPane().add(totalLabel = new JLabel("0"));

	comboAktie.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    if (!buy) {
			int oldAnzahl = -1;
			int wkn = ((JbAktie)(aktienHM.get((String)(comboAktie.getSelectedItem())))).getWKN();

			ResultSet res1 = JbSQL.query("select Anzahl from DEPOTINHALT where (WKN=\'"+wkn+
						     "\' and DepotID=\'"+depotID+"\')");
			try {
			    while (res1.next())
				oldAnzahl = new Integer((String)(res1.getString(1).trim())).intValue();
			}catch(SQLException sqle){sqle.printStackTrace();};

			anzahl.setText(new Integer(oldAnzahl).toString());
			anzahl.selectAll();
		    }

		    kursLabel.setText(new Double(((JbAktie)(aktienHM.get((String)(comboAktie.getSelectedItem())))).getAktKurs()).toString());
		    totalLabel.setText(new Double(Double.parseDouble(kursLabel.getText().trim())*
						  Double.parseDouble(anzahl.getText().trim())).toString());

		    total = Double.parseDouble(totalLabel.getText().trim());

		    kursLabel.setText(JbDefinition.betragEUR_DM(kursLabel.getText()));
		    totalLabel.setText(JbDefinition.betragEUR_DM(totalLabel.getText()));
   		}});
	comboAktie.setSelectedIndex(0);

	JButton okButton = new JButton("Na gut");
	okButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    createBuy();
   		}});
	JButton cancelButton = new JButton("Vergiss es!");
	cancelButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e){
		    setVisible(false);
		    dispose();
   		}});
	getContentPane().add(okButton);
	getContentPane().add(cancelButton);
    }

    /**
     * OK pressed (handle the transactions)
     */
    private void createBuy() {
	kursLabel.setText(new Double(((JbAktie)(aktienHM.get((String)(comboAktie.getSelectedItem())))).getAktKurs()).toString());
	try {
	    totalLabel.setText(new Double(Double.parseDouble(kursLabel.getText().trim())*
					  Double.parseDouble(anzahl.getText().trim())).toString());
	    
	    total = Double.parseDouble(totalLabel.getText().trim());

	    // get the 0.7% for MakeMonkey
	    double aux = 0.007*total;
	    if(aux<15)
		aux=15.;
	    if (buy)
		total = aux+total;
	    else
		total = -aux+total;

	    kursLabel.setText(JbDefinition.betragEUR_DM(kursLabel.getText()));
	    totalLabel.setText(JbDefinition.betragEUR_DM(totalLabel.getText()));
	} catch(Exception ex) {};

	int handelID = 0;

	//	comboAktie.setSelectedItem(comboAktie.getSelectedItem());

	ResultSet res = JbSQL.query("select nvl(max(HandelID)+1, 0) from HANDEL");
	try {
	    if (res.next())
		handelID = new Integer((String)(res.getString(1).trim())).intValue();
	}catch(SQLException e){e.printStackTrace();};


	int wkn = ((JbAktie)(aktienHM.get((String)(comboAktie.getSelectedItem())))).getWKN();
	double aktKurs = ((JbAktie)(aktienHM.get((String)(comboAktie.getSelectedItem())))).getAktKurs();
	int myAnzahl;
	try {
	    myAnzahl = Integer.parseInt(anzahl.getText());
	}catch(NumberFormatException e){
	    anzahl.setText("100");
	    anzahl.selectAll();
	    return;
	}

	setVisible(false);

	if (!buy)
	    myAnzahl = -myAnzahl;

	int oldAnzahl = -1;
	res = JbSQL.query("select Anzahl from DEPOTINHALT where (WKN=\'"+wkn+
			  "\' and DepotID=\'"+depotID+"\')");
	try {
	    while (res.next())
		oldAnzahl = new Integer((String)(res.getString(1).trim())).intValue();
	}catch(SQLException e){e.printStackTrace();};
	int newAnzahl = oldAnzahl + myAnzahl;


	double betrag = JbSQL.getKundeKontoStand(kontoID);
	double kredit = JbSQL.getKundeKreditLimit_EUR(depotID, kontoID);

	if (buy)
	    total = -total;

	if ((betrag + kredit + total < 0) && buy) {
	    JOptionPane.showMessageDialog(this,
					  "Nicht genug Geld dafuer!",
					  "Kaufen", JOptionPane.ERROR_MESSAGE);
	    return;
	}

	JbSQL.query("insert into HANDEL values (\'"+wkn+"\', \'"+
		    kundeID+"\', \'"+handelID+"\', \'"+
		    JbSQL.today()+"\', "+aktKurs+", "+
		    myAnzahl+")");

	if ((oldAnzahl != -1) && (newAnzahl == 0))
	    JbSQL.query("delete from DEPOTINHALT where (DepotID=\'"+depotID+
			"\' and WKN=\'"+wkn+"\')");
	else
	    if (newAnzahl < 0) {
		JOptionPane.showMessageDialog(this,
					      "Nicht so vielle Aktien!",
					      "Verkaufen", JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    else
		if (oldAnzahl != -1)
		    JbSQL.query("update DEPOTINHALT set Anzahl="+newAnzahl+
				" where (DepotID=\'"+depotID+"\' and WKN=\'"+wkn+"\')");
		else
		    JbSQL.query("insert into DEPOTINHALT values (\'"+depotID+"\', \'"+wkn+"\', "+myAnzahl+")");
		
	betrag += total;
	JbSQL.query("update KKONTO set Betrag="+JbDefinition.sprintf(betrag,5)+" where kontoID=\'"+kontoID+"\'");
		
	dispose();
    }
}
