package Jbroker;


import java.io.*;
import java.sql.*;
import java.util.*;
import java.text.*;
import javax.swing.*;

/**
 * A SQL related work utility class.
 * @version Java Fast Broker 1.0
 * @author Valentin Zaharescu & Bogdan Stanca<br>
 * Copyright (c)2000 all rights reserved<br>
 * Abstract class  to handle the SQL-related actions
 */
public abstract class JbSQL {
    private static Connection aConnection;   // Neccessary for the connection

    // URL-of database
    private static final String DbUrl = "jdbc:oracle:thin:db1/Morcov@pcha2.physik.fu-berlin.de:1521:orcl";

    // attributes keeping the open connection and a statement
    private static Statement aStatement_;

    /** executes a query and returns the resultset
     *
     * @param SQLQuery the query to be executed
     * @return the result set returned by the database
     */
    public static ResultSet query(String aSQLQuery){ 
	ResultSet rs=null;
	try{
	    rs=aStatement_.executeQuery(aSQLQuery);
	}catch (SQLException e){
	    e.printStackTrace ();
	}
	return  rs;
    }

    /** executes an update and returns the count of affected rows
     *
     * @param SQLQuery the update to be executed
     * @return the count of affected rows
     * @throws java.sql.SQLException all exceptions of the dbdriver are thrown
     */
    public static int update(String aSQLUpdate) throws SQLException {
	return aStatement_.executeUpdate(aSQLUpdate);
    }
  
    /**
     *  opens the connection
     */
    public static void open() {
	System.out.println ("Trying to connect to database....\n"); 

	try
	    {
		Class.forName ("oracle.jdbc.driver.OracleDriver");
		aConnection = DriverManager.getConnection (DbUrl);

		aStatement_ = aConnection.createStatement (); 
		System.out.println ("   Connection ok!\n");
	    }
	catch (Exception e)
	    {
		System.out.println("Server not available.\nPlease contact your nearest Oracle administrator and dial again.");
		System.exit(-1);
	    }
    }

    /** 
     *  closes the connection
     */
    public static void close() {
	try{
	    aStatement_.close();
	} catch (SQLException e) { 
	    System.out.println("DBAccess.constuctor: " + e.getMessage());
	}  
    }

    /*
     * converts Calendar-Date to Date in SQL Format
     */
    public static String toSQLDate(Calendar cal) {
	DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM);
	return cal.get(Calendar.DAY_OF_MONTH)+"-"+
	    df.format(cal.getTime()).substring(0, 3).toUpperCase()+"-"+
	    cal.get(Calendar.YEAR);
    }

    /*
     * returns today's Calendar-Date to Date in SQL Format
     */
    public static String today() {
	String nextVirtualDate = new String();
	try
	    { // get last virtual date
		ResultSet Res = JbSQL.query("select TO_CHAR(max(Datum)+1, 'DD-MON-YYYY') from kurs");
		if (Res.next())
		    nextVirtualDate = Res.getString(1).trim();
	    }catch(Exception e) {e.printStackTrace();}
	return nextVirtualDate;
    }

    /*
     * creates a JComboBox with all the Names from Aktie
     */
    public static JComboBox createAktienJComboBox() {
	JComboBox tmpCombo = new JComboBox();

	ResultSet res = JbSQL.query("select Name from AKTIE order by Name");
	try {
	    while (res.next())
		tmpCombo.addItem(res.getString(1).trim());
	}catch(SQLException e){e.printStackTrace();};

	return tmpCombo;
    }

    /*
     * creates a JComboBox with all the Names from Aktie
     */
    public static JComboBox createOwnAktienJComboBox(int depotID) {
	JComboBox tmpCombo = new JComboBox();

	ResultSet res = JbSQL.query("select a1.Name from AKTIE a1, "+
				    "(select d.WKN from DepotInhalt d where d.DepotID=\'"+depotID+
				    "\' ) a2 where a1.WKN=a2.WKN order by Name");
	try {
	    while (res.next())
		tmpCombo.addItem(res.getString(1).trim());
	}catch(SQLException e){e.printStackTrace();};

	return tmpCombo;
    }

    /*
     * creates a HashMap with all the Names (as keys for JComboBox'es)
     * and JbAktie as details from Aktie
     */
    public static HashMap createAktienHashMap() {
	HashMap aktienHM = new HashMap();

	ResultSet res = JbSQL.query("select Name, WKN, AktKurs, MinKurs, MaxKurs,"+
				    " Volumen from AKTIE");
	try {
	    while (res.next())
		aktienHM.put((String)(res.getString(1).trim()),
			     new JbAktie(res.getString(1), res.getString(2), res.getString(3),
					 res.getString(4), res.getString(5), res.getString(6)));
	}catch(SQLException e){e.printStackTrace();};

	return aktienHM;
    }

    /*
     * creates a HashMap indexed after counter
     * and JbAktie as details from Aktie
     */
    public static HashMap getAktienHM() {
	HashMap aktienHM = new HashMap();

	ResultSet res = JbSQL.query("select Name, WKN, AktKurs, MinKurs, MaxKurs,"+
				    " Volumen from AKTIE");
	int count = 0;
	try {
	    while (res.next())
		aktienHM.put(new Integer(count++),
			     new JbAktie(res.getString(1), res.getString(2), res.getString(3),
					 res.getString(4), res.getString(5), res.getString(6)));
	}catch(SQLException e){e.printStackTrace();};

	return aktienHM;
    }

    /*
     * @returns the WKN Aktie Name
     */
    public static String getAktieName(int wkn) {
	String name = new String("unknown");
	ResultSet res = JbSQL.query("select Name from AKTIE where (wkn="+wkn+")");
	try {
	    if (res.next())
		name =(String)(res.getString(1).trim());
	}catch(SQLException e){e.printStackTrace();};
	return name;
    }

    /*
     * @returns the WKN Aktie Kurs
     */
    public static double getAktieKurs(int wkn) {
	double kurs = 0.0;
	ResultSet res = JbSQL.query("select AktKurs from AKTIE where (wkn="+wkn+")");
	try {
	    if (res.next())
		kurs = Double.parseDouble(res.getString(1).trim());
	}catch(SQLException e){e.printStackTrace();};
	return kurs;
    }

    /*
     * @returns the WKN Aktie First Kurs
     */
    public static double getAktieOldKurs(int wkn) {
	double kurs = 0.0;
	ResultSet res = JbSQL.query("select TagesKurs from KURS where (wkn="+wkn+") order by Datum asc");
	try {
	    if (res.next())
		kurs = Double.parseDouble(res.getString(1).trim());
	}catch(SQLException e){e.printStackTrace();};
	return kurs;
    }

    /*
     * creates a HashMap with all the Names (as keys for JComboBox'es)
     * and JbAktie as details from Aktie
     */
    public static HashMap createOwnAktienHashMap(int depotID) {
	HashMap aktienHM = new HashMap();

	ResultSet res = JbSQL.query("select a1.Name, a1.WKN, a1.AktKurs, a1.MinKurs, "+
				    "a1.MaxKurs, a1.Volumen from AKTIE a1, "+
				    "(select d.WKN from DepotInhalt d where d.DepotID=\'"+depotID+
				    "\' ) a2 where a1.WKN=a2.WKN");
	try {
	    while (res.next())
		aktienHM.put((String)(res.getString(1).trim()),
			     new JbAktie(res.getString(1), res.getString(2), res.getString(3),
					 res.getString(4), res.getString(5), res.getString(6)));
	}catch(SQLException e){e.printStackTrace();};

	return aktienHM;
    }

    /*
     * creates a HashMap with all the (Name+" "+Vorname) (as keys for the JbUserList)
     * as details from Kunde
     */
    public static HashMap createKundenHashMap()
    {
	HashMap kundenHM = new HashMap();

    	ResultSet Res = JbSQL.query("select Name, Vorname, kID from KUNDE");
	try{
	    while(Res.next())
		kundenHM.put(Res.getString(1).trim()+" "+Res.getString(2).trim(), Res.getString(3).trim());
	}catch(SQLException e){e.printStackTrace();}

    	return kundenHM;
    }

    /*
     * creates a JList with all the (Name+" "+Vorname)
     * as details from Kunde
     */
    public static JList createKundenJList()
    {
	TreeSet kundenTS = new TreeSet();

    	ResultSet Res = JbSQL.query("select Name, Vorname, kID from KUNDE");
	try{
	    while(Res.next())
		kundenTS.add(Res.getString(1).trim()+" "+Res.getString(2).trim());
	}catch(SQLException e){e.printStackTrace();}

	String[] result = new String[kundenTS.size()];

	int i=0;
	Iterator it = kundenTS.iterator();
	while (it.hasNext())
	    result[i++] = (String)it.next();

    	JList userList = new JList(result);

    	return userList;
    }

    /*
     * returns the value of the kunde current shares stock
     */
    public static double getKundeAktienWert(int depotID)
    {
	double kredit = 0;
	try
	    { // Query depot for valueOfDepot
		ResultSet Res = JbSQL.query("select a2.Anzahl, a1.AktKurs from aktie a1, "+
					    "(select d.WKN, sum(d.Anzahl) Anzahl "+
					    "from DepotInhalt d where d.DepotID=\'"+depotID+"\' group by d.WKN) a2 "+
					    "where a1.WKN=a2.WKN order by a1.Name");
		while(Res.next())
		    kredit += Double.parseDouble(Res.getString(1).trim()) *
			Double.parseDouble(Res.getString(2).trim());
	    }catch(Exception e) {e.printStackTrace();}

	return kredit;
    }

    /*
     * returns the value of the kunde current credit limit in EURO
     */
    public static double getKundeKreditLimit_EUR(int depotID, int kontoID)
    {
	double kredit = getKundeKreditLimit_prozent(kontoID)/100.0*
	    getKundeAktienWert(depotID);

	return kredit;
    }

    /*
     * returns the value of the kunde current credit limit in percent
     */
    public static double getKundeKreditLimit_prozent(int kontoID)
    {
	double kredit = 0;

	try
	    { // get KontoStand & IndivKredit
		ResultSet Res = JbSQL.query("select IndivKredit from kkonto where kontoID=\'"+kontoID+"\'");
		while(Res.next())
		    kredit = Double.parseDouble(Res.getString(1).trim());
	    }catch(Exception e) {e.printStackTrace();}

	return kredit;
    }

    /*
     * returns the current value of the kunde's bank account
     */
    public static double getKundeKontoStand(int kontoID)
    {
	double betrag = 0;

	try
	    { // get KontoStand & IndivKredit
		ResultSet Res = JbSQL.query("select Betrag from kkonto where kontoID=\'"+kontoID+"\'");
		while(Res.next())
		    betrag = Double.parseDouble(Res.getString(1).trim());
	    }catch(Exception e) {e.printStackTrace();}

	return betrag;
    }


    /*
     * updates the current stock to a new value (let's say from a stock server)
     */
    public static void setNewKurs()
    {
	String nextVirtualDate = new String();
	HashMap aktien = new HashMap();
	HashMap aktienWKN = new HashMap();
	int countAktienWKN = 0;

	try
	    { // get last virtual date
		ResultSet Res = JbSQL.query("select to_CHAR(max(Datum),\'DD-MON-YYYY\') from kurs");
		if (Res.next())
		    nextVirtualDate = Res.getString(1).trim();
		
		// get WKN and Kurs
		Res = JbSQL.query("select TagesKurs, WKN from kurs where(Datum=\'"+nextVirtualDate+"\')");
		while (Res.next()) {
		    aktienWKN.put(new Integer(countAktienWKN), new Integer(Res.getString(2).trim()));
		    aktien.put( aktienWKN.get(new Integer(countAktienWKN)),  new Double(Res.getString(1).trim()));
		    countAktienWKN++;
		}
		// get last virtual date
		Res = JbSQL.query("select to_CHAR(max(Datum)+1,\'DD-MON-YYYY\') from kurs");
		if (Res.next())
		    nextVirtualDate = Res.getString(1).trim();

		for  (int i=0;i<countAktienWKN;i++) {
		    double newKurs = JbDefinition.newRandomKurs((Double)(aktien.get((Integer)(aktienWKN.get(new Integer(i))))), (Integer)(aktienWKN.get(new Integer(i))));
		    if (newKurs < 0)
			newKurs = 2.32*Math.random();

		    JbSQL.query("insert into kurs values(\'"+nextVirtualDate+"\', "+newKurs+", "+aktienWKN.get(new Integer(i))+")");
		    JbSQL.update("update aktie set AktKurs="+newKurs+", MinKurs="+(newKurs-Math.random()*0.37)+", MaxKurs="+(newKurs+Math.random()*0.42)+" where WKN="+aktienWKN.get(new Integer(i)));
		}
	    }catch(Exception e) {e.printStackTrace();}

    }

    /*
     * returns true if no Aktien were found in Depot
     */
    public static boolean keineAktienImDepot(int depotID) {
	boolean result = true;

	ResultSet res = JbSQL.query("select WKN from DepotInhalt where DepotID=\'"+depotID+"\'");
	try {
	    if (res.next())
		result = false;
	}catch(SQLException e){e.printStackTrace();};

	return result;
    }
}
