package Jbroker;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import java.sql.*;

/**
 * Main Window
  * @version Java Fast Broker 1.0
 * @author Valentin Zaharescu & Bogdan Stanca<br>
 * Copyright (c)2000 all rights reserved<br>
*/
public class JbUserDialog extends JFrame
{
    private final JButton loginButton=new JButton("Anmelden");
    private final JButton logoutButton = new JButton("Abmelden");
    private final JButton addButton = new JButton("Erstellen");
    private final JButton removeButton = new JButton("Lschen");
    private final JButton editButton = new JButton("Bearbeiten");
    private final JButton quitButton = new JButton("Weg damit");

    private final JPanel buttonPanel = new JPanel(new GridLayout(1, 6));

    private HashMap userHM=null;
    private JList userLoginList=null;
    private JScrollPane userLoginPane;

    private JbTable table;
    private JbMainPanel actionPanel;

    final private JPanel thisPanel = (JPanel)this.getContentPane(); 
    final private JPanel northPanel = new JPanel(new BorderLayout()) ; 

    private Border lineBorder = null;

    private int lastLoginID = 0;
    private String lastLogin=null;

    /**
     * Constructor
     */
    public JbUserDialog()
    {
        super("Java Fast Broker - Klient Dialog");

	// Set layout manager.
        getContentPane().setLayout(new BorderLayout());

	logoutButton.setEnabled(false);

        buttonPanel.add(quitButton);
        buttonPanel.add(loginButton);
        buttonPanel.add(logoutButton);
        buttonPanel.add(addButton);
        buttonPanel.add(editButton );
        buttonPanel.add(removeButton);

	lineBorder = BorderFactory.createLineBorder(Color.blue, 2);
	northPanel.add(buttonPanel, BorderLayout.NORTH);
	northPanel.setBorder(lineBorder);

	createUserLoginList();
	userLoginPane=new JScrollPane(userLoginList);

	getContentPane().add(northPanel, BorderLayout.NORTH);
	getContentPane().add(userLoginPane, BorderLayout.CENTER);

        quitButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    setVisible(false);
		    dispose();

		    // close the connection
		    JbSQL.close();

		    System.exit(0);
		}
	    });

        loginButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    if (!checkUserLogin())
			return;

		    loginButton.setEnabled(false);
		    addButton.setEnabled(false);
		    editButton.setEnabled(false);
		    removeButton.setEnabled(false);
		    logoutButton.setEnabled(true);

		    // Retrieve database and model.
		    thisPanel.remove(userLoginPane);

		    //create and add panel
		    actionPanel = new JbMainPanel(lastLoginID);
		    thisPanel.add(actionPanel, BorderLayout.CENTER);

		    pack();
		    repaint();
		}
	    });

        logoutButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    loginButton.setEnabled(true);
		    addButton.setEnabled(true);
		    editButton.setEnabled(true);
		    removeButton.setEnabled(true);
		    logoutButton.setEnabled(false);

		    thisPanel.remove(actionPanel);
		    actionPanel.deleteProcesses();

		    createUserLoginList();
		    userLoginPane=new JScrollPane(userLoginList);
		    thisPanel.add(userLoginPane, BorderLayout.CENTER);
		    pack();
		    repaint();

		    JbSQL.setNewKurs();
		}
	    });

        addButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    // Show user edit dialog.

		    JbUserEditDialog dlg = new JbUserEditDialog(
								JOptionPane.getFrameForComponent(JbUserDialog.this), true,
								"Neuer Klient erstellen");
		    centerWindow(dlg);
		    dlg.show();

		    if (dlg.isOkay()){
		    
			String name = dlg.getFirstName().trim();
			String vorname = dlg.getLastName().trim();

			int kID = -1;
			int konto = -1;
			int depot = -1;
			ResultSet res = JbSQL.query("select DepotID, KontoID, kID from KUNDE");
			try {
			    while (res.next()) {
				int aux = new Integer((String)(res.getString(1).trim())).intValue();
				if (depot < aux)
				    depot = aux;
				aux = new Integer((String)(res.getString(2).trim())).intValue();
				if (konto < aux)
				    konto = aux;
				aux = new Integer((String)(res.getString(3).trim())).intValue();
				if (kID < aux)
				    kID = aux;
			    }
			}catch(SQLException e){e.printStackTrace();};
			depot++;	konto++;	kID++;

			JbSQL.query("insert into kkonto values ("+konto+", 0, 50)");
			JbSQL.query("insert into depotkonto values (\'"+depot+"\', \'"+JbSQL.today()+"\')");
			JbSQL.query("insert into kunde values (\'"+kID+"\', \'"+name+"\', \'"+
				    vorname+"\',"+ depot+", "+konto+")");
			lastLogin = name+" "+vorname;
			JOptionPane.showMessageDialog(JbUserDialog.this,
						      "Der Kunde \'" + lastLogin + "\' \nwurde erstellt!\n"+
						      "Der Password (KID) ist \'"+kID+"\' !!!!",
						      "Delete", JOptionPane.ERROR_MESSAGE);

			updateUserJList();
		    }

		}
	    });

        editButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    if (!checkUserLogin())
			return;

		    // Show user edit dialog.
		    JbUserEditDialog dlg = new JbUserEditDialog(
								JOptionPane.getFrameForComponent(JbUserDialog.this), false,
								"Klient bearbeiten");
		    centerWindow(dlg);
		    dlg.setEmail(lastLogin);

		    String firstName = new String();
		    String lastName = new String();
		    ResultSet res = JbSQL.query("select Name, Vorname from KUNDE where kID="+lastLoginID);
		    try {
			while (res.next()) {
			    firstName = (String)(res.getString(1).trim());
			    lastName = (String)(res.getString(2).trim());
			}
		    }catch(SQLException e){e.printStackTrace();};

		    dlg.setFirstName(firstName);
		    dlg.setLastName(lastName);
		    dlg.show();

		    firstName = dlg.getFirstName();
		    lastName = dlg.getLastName();

		    if (dlg.isOkay()) {
			JbSQL.query("update KUNDE set Name=\'"+firstName+"\', Vorname=\'"+
				    lastName+"\' where kID="+lastLoginID);

			updateUserJList();
		    }
		}
	    });

        removeButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    if (!checkUserLogin())
			return;

		    int konto = -1;
		    int depot = -1;
		    ResultSet res = JbSQL.query("select DepotID, KontoID from KUNDE");
		    try {
			while (res.next()) {
			    depot = new Integer((String)(res.getString(1).trim())).intValue();
			    konto = new Integer((String)(res.getString(2).trim())).intValue();
			}
		    }catch(SQLException e){e.printStackTrace();};

		    if ((konto == -1) || (depot == -1)) {
			JOptionPane.showMessageDialog(JbUserDialog.this,
						      "Der Klient \'" + lastLogin + "\' \nkonnte nicht gelscht werden!",
						      "Lschen", JOptionPane.ERROR_MESSAGE);
			return;
		    }

		    JbSQL.query("delete from handel where kID="+lastLoginID);
		    JbSQL.query("delete from kunde where kID="+lastLoginID);

		    JbSQL.query("delete from kkonto where KontoID="+konto);

		    JbSQL.query("delete from depotinhalt where DepotID="+depot);
		    JbSQL.query("delete from depotkonto where DepotID="+depot);

		    updateUserJList();

		    JOptionPane.showMessageDialog(JbUserDialog.this,
						  "Der Klient \'" + lastLogin + "\' wurde gelscht!",
						  "Lschen", JOptionPane.ERROR_MESSAGE);
		}});
    }


    /**
     * show User Menubar NotActive Error
     */
    void createUserLoginList()
    {
	if (userLoginList != null) {
	    userLoginList.setVisible(false);
    	}
	userHM = JbSQL.createKundenHashMap();
	userLoginList = JbSQL.createKundenJList();
	userLoginList.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    if (e.getClickCount() >= 1) {
			int index = userLoginList.locationToIndex(e.getPoint());
			if (index != -1) {
			    userLoginList.setSelectedIndex(index);
			    lastLogin=(String)userLoginList.getSelectedValue();
			    lastLoginID=Integer.parseInt((String)(userHM.get((String)userLoginList.getSelectedValue())));
			}
		    }
		    if (e.getClickCount() == 2)
			loginButton.doClick();
		}
			
	    });
    }

    /**
     * Utility login function.
     */
    public boolean checkUserLogin()
    {
	// Show authentification dialog.
	JbAuthentificationDialog dlg = new JbAuthentificationDialog(
								    JOptionPane.getFrameForComponent(JbUserDialog.this));
	dlg.setEmail(lastLogin);
	centerWindow(dlg);

	dlg.show();

	// If the user clicked okay, then try to log in
	if (dlg.isOkay())
	    {
		lastLogin = dlg.getEmail();
		lastLoginID = dlg.getPassword();

		if(!verifyUser(lastLogin, new Integer(lastLoginID).toString())) {
		    JOptionPane.showMessageDialog(JbUserDialog.this,
						  "Kunde nicht erstellt oder falsches Kennwort!",
						  "Klientanmeldung", JOptionPane.ERROR_MESSAGE);
		    return false; 
		}
	    } 
	else
	    return false;

	return true;
    }

    /**
     * Check login and password.
     */
    private boolean verifyUser(String name, String passwd)
    {
    	if (userHM.containsKey(name.trim()))
	    return ((String)(userHM.get(name.trim()))).equals(passwd.trim());
	return false;
    }

    /**
     * Utility function to center window.
     */
    public void centerWindow(Component window)
    {
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Dimension dim = toolkit.getScreenSize();
        int x = (dim.width - window.getSize().width) / 2;
        int y = (dim.height - window.getSize().height) / 2;
        window.setLocation(x, y);
    }

    /**
     * updates the user JList.
     */
    public void updateUserJList()
    {
	thisPanel.remove(userLoginPane);
	createUserLoginList();
	userLoginPane=new JScrollPane(userLoginList);
	thisPanel.add(userLoginPane, BorderLayout.CENTER);
	pack();
	repaint();
    }
}
