package Jbroker;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A dialog box to add/edit users properties.
 * @version Java Fast Broker 1.0
 * @author Valentin Zaharescu & Bogdan Stanca<br>
 * Copyright (c)2000 all rights reserved<br>
 */
public class JbUserEditDialog 
    extends JDialog
{
    private JPanel mainPanel = null;

    private JTextField emailField  = null;
    private JTextField firstNameField  = null;
    private JTextField lastNameField  = null;

    private JButton okayButton = null;
    private JButton cancelButton = null;

    private boolean result = false;
    private boolean addingUser = false;


    /**
     * Constructor
     */
    public JbUserEditDialog(Frame parent, boolean isAddUser, String frameName)
    {
        super(parent, frameName, true);

        addingUser = isAddUser;

	// Dialog is not resizable.
        setResizable(false);
	setSize(300, 250);
	setModal(true);
	mainPanel = new JPanel() {
    		public  void paintComponent(Graphics g) {
		    if (g != null) {
			ImageIcon tmpImage = new ImageIcon(JbDefinition.AddNewUserImage);
			g.drawImage(tmpImage.getImage(), 0, 0, mainPanel.getWidth(), mainPanel.getHeight(), this);
			super.paintComponent(g);
		    }
		}};
	mainPanel.setOpaque(false);

        // Set close handler for window.
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent event)
		{
		    doCancel();
		}
	    });


        // Layout components.
        getContentPane().add(mainPanel);
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(createEditPanel(), BorderLayout.NORTH);
        mainPanel.add(createButtonPanel(), BorderLayout.SOUTH);
    }


    /**
     * Return the result of the dialog.
     */
    public boolean isOkay()
    {
        return result;
    }

    /**
     * Returns the login name
     */
    public String getEmail()
    {
        return emailField.getText();
    }

    /**
     * Set login name
     */
    public void setEmail(String email)
    {
        emailField.setText(email);
    }


    /**
     * Return the firstName.
     */
    public String getFirstName()
    {
        return firstNameField.getText();
    }

    /**
     * Set the firstName.
     */
    public void setFirstName(String fn)
    {
        firstNameField.setText(fn);
    }


    /**
     * Return the lastName.
     */
    public String getLastName()
    {
        return lastNameField.getText();
    }

    /**
     * Set the lastName.
     */
    public void setLastName(String ln)
    {
        lastNameField.setText(ln);
    }


    /**
     * Closes the dialog with "true" result.
     */
    protected void doOkay()
    {
	result = true;
	setVisible(false);
	dispose();
    }

    /**
     * Closes the dialog with "false" result.
     */
    protected void doCancel()
    {
        result = false;
        setVisible(false);
        dispose();
    }

    /**
     * Creates the edit panel.
     */
    protected JPanel createEditPanel()
    {
	JPanel panel = new JPanel(new GridLayout(3,2));
	panel.setOpaque(false);

	JLabel ulabel, fnlabel, lnlabel;

	panel.add(ulabel = new JLabel("Klient:"));
	ulabel.setForeground(Color.white);
	ulabel.setHorizontalAlignment(JLabel.CENTER);
	panel.add(emailField = new JTextField(20));
	emailField.setEditable(false);

	panel.add(fnlabel = new JLabel("Name:"));
	fnlabel.setForeground(Color.white);
	fnlabel.setHorizontalAlignment(JLabel.CENTER);
	panel.add(firstNameField = new JTextField(20) {
		public void actionPerformed(ActionEvent event)
		{
		    lastNameField.requestFocus();
		}
	    });

	panel.add(lnlabel = new JLabel("Vorname:"));
	lnlabel.setForeground(Color.white);
	lnlabel.setHorizontalAlignment(JLabel.CENTER);
	panel.add(lastNameField = new JTextField(20));

	return panel;
    }


    /**
     * Creates the button panel.
     */
    protected JPanel createButtonPanel()
    {
        JPanel panel = new JPanel();
	panel.setOpaque(false);
	String okText = null;
	if (addingUser)
	    okText = new String("Einfgen");
	else
	    okText = new String("Na gut");

        panel.add(okayButton = new JButton(okText));
        panel.add(cancelButton = new JButton("Spter"));

        // Add some action listeners.
        okayButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    doOkay();
		}
	    });

        cancelButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event)
		{
		    doCancel();
		}
	    });

        return panel;
    }
}
